#ifndef ANIMBITM_H

#define ANIMBITM_H

#include <stddef.h>
#include "stddefs.h"
#include "yakList.h"
#include "yakbitm.h"

//@doc

/*
@class animBitmaps are animated <c yakBitmap>s.  Their linked-list structure allows easy transition
between animation frames.

@base public | pyList
*/
class animBitmap : public pyList
{
public:
  //@access Public Members
  //@cmember Iterator for the current frame of animation
  ptListIterator<yakBitmap> tIThisFrame;
  //@member ~animBitmap | Default destructor
  virtual ~animBitmap() {};
  //@cmember Default constructor
  animBitmap();
  //@cmember Specific constructor (loads all frames from disk)
  animBitmap( const char * CszFilename, yakLib * plib = NULL, yakBitmap::bitmapType flags = yakBitmap::normal );
  //@cmember Adds a frame from an existing <c yakBitmap>
  bFlag bfAddFrameFromBitmap( yakBitmap * pybmNewFrame );
  //@cmember Adds a frame from disk
  bFlag bfAddFrame( const char * CszFilename, yakLib * plib = NULL, yakBitmap::bitmapType flags = yakBitmap::normal );
  //@cmember Adds all frames from disk
  bFlag bfAddAllFrames( const char * CszFilename, yakLib * plib = NULL, yakBitmap::bitmapType flags = yakBitmap::normal );
  //@member Advance | Advances a frame of animation
  inline virtual void Advance(void) { ++tIThisFrame; };
  //@cmember Advances a specific number of frames of animation
  virtual void Advance( int iNumFrames );
  //@member Show | Shows the current frame of animation
  inline void Show( pageHandle_t pagPage, screenCoordX_t scx, screenCoordY_t scy, yakBitmap::showType eShowType = yakBitmap::masked ) const { tIThisFrame.ptCurrent()->Show( pagPage, scx, scy, eShowType ); };
  //@member ShowAndAdvance | Shows the current frame, then advances one frame of animation
  inline void ShowAndAdvance( pageHandle_t pagPage, screenCoordX_t scx, screenCoordY_t scy, yakBitmap::showType eShowType = yakBitmap::masked ) { tIThisFrame.ptCurrent()->Show( pagPage, scx, scy, eShowType ); Advance(); };
  //@member ShowCollision | Shows the animBitmap in a solid color
  inline void ShowCollision( pageHandle_t pagPage, screenCoordX_t scx, screenCoordY_t scy, color_t colColor ) const { tIThisFrame.ptCurrent()->ShowCollision( pagPage, scx, scy, colColor ); };
  //@member ShowZoomed | Shows the animBitmap zoomed to new dimensions
  inline void ShowZoomed( pageHandle_t pagPage, screenCoordX_t scx, screenCoordY_t scy, int iNewWidth,
    int iNewHeight, yakBitmap::showType eShowType = yakBitmap::masked ) const { tIThisFrame.ptCurrent()->ShowZoomed( pagPage, scx, scy, iNewWidth, iNewHeight, eShowType ); };
  //@cmember Covers bitmap-sized region with data from another page of video memory
  void Hide( pageHandle_t pagPageTo, screenCoordX_t scx, screenCoordY_t scy, pageHandle_t pagFrom ) const;
};

/*
@class An animslave performs as an <c animBitmap> but instead of holding the actual frames,
it just holds a pointer to the master <c animBitmap>.  This allows it to be a much smaller
structure.

@base public | ptListIterator
*/
class animslave : public ptListIterator<yakBitmap>
{
public:
  //@access Public Members
  //@cmember Default constructor
  animslave();
  //@cmember Specific constructor, initializes animslave to an <c animBitmap>
  animslave(animBitmap * panmMaster);
  //@cmember default destructor
  virtual ~animslave();
  //@member Advance | Advances the animslave a frame of animation
  inline virtual void Advance( void ) { ++( *this ); };
  //@cmember Advances a given number of frames of animation
  virtual void Advance( int iNumFrames );
  //@member Show | Shows the current animation frame
  inline void Show( pageHandle_t pagPage, screenCoordX_t scx, screenCoordY_t scy, yakBitmap::showType eShowType = yakBitmap::masked ) const { ptCurrent()->Show( pagPage, scx, scy, eShowType ); };
  //@member ShowAndAdvance | Shows the current animation frame then advances one animation frame
  inline void ShowAndAdvance( pageHandle_t pagPage, screenCoordX_t scx, screenCoordY_t scy, yakBitmap::showType eShowType = yakBitmap::masked ) { ptCurrent()->Show( pagPage, scx, scy, eShowType ); Advance(); };
  //@member ShowCollision | Shows the animslave in a solid color
  inline void ShowCollision( pageHandle_t pagPage, screenCoordX_t scx, screenCoordY_t scy, color_t colColor ) const { ptCurrent()->ShowCollision( pagPage, scx, scy, colColor ); };
  //@member ShowZoomed | Shows the current animation frame zoomed to new dimensions
  inline void ShowZoomed( pageHandle_t pagPage, screenCoordX_t scx, screenCoordY_t scy, int iNewWidth,
    int iNewHeight, yakBitmap::showType eShowType = yakBitmap::masked ) const { ptCurrent()->ShowZoomed( pagPage, scx, scy, iNewWidth, iNewHeight, eShowType ); };
};

#endif
