#ifndef EVENT_H

#define EVENT_H

#include "actor.h"

#define MAX_EVENTS 10
//@doc

/*
@enum event::eventType | Type of event
@emem scriptLimited | Event removes itself at end of script
@emem animationlimited | Event removes itself at end of animation
*/

/*
@class Events are single-run actors which remove themselves from their <c animap> after
their animation ends or their script ends, dependent on their event type.

@base public | actor
*/
class event :
  public actor {
protected:
  //@access Protected Members
  //@cmember Returns "live" or "dead" status of the event.
  bFlag bfIsStillAlive( void );
  //@cmember Removes the event from the map and destroys it.
  void Die( void );
public:
  //@access Public Members
  //@cmember Default Constructor
  event( void );
  //@cmember Copy constructor
  event( const event * pevnCopyFrom );
  //@cmember default destructor
  virtual ~event( void );
  enum eventType {
    scriptLimited,    //removes itself at end of script
    animationLimited  //removes itself when animation ends
  };
  //@cmember Type of event
  eventType eEventType;
  //@cmember Advances the event
  virtual void Advance( void );
  //@cmember Adds a copy of this event to a given position on an <c animap>
  void AddCopyAt( animap * pmap, mapCoordX_t mcx, mapCoordY_t mcy, eventType eNewEventType ) const;
};
#endif
