#ifndef FACTOR_H

#define FACTOR_H

#include "actor.h"

#define NUM_FACES 10
#define CENTER_FACE 5
#define DEFAULT_ID_ARG 255

//@doc

/*
@class This is an array of animBitmap representing the different animated "faces" of a
<c facingActor>.
*/
class directionTable {
public:
  //@access Public Members
  //@cmember Array of <c animBitmap>s for facing
  animBitmap aanmFaces[ NUM_FACES ];
  //@member Load | loads the images into the direction table.
  void Load( char * pcSpecial, char * pcSw, char * pcS, char * pcSe,
    char * pcW, char * pcCenter, char * pcE, char * pcNw, char * pcN,
    char * pcNe,  yakLib * plib = NULL,
    yakBitmap::bitmapType eNewBitmapType = yakBitmap::normal );
  //@cmember Frees memory owned by the directionTable
  void FreeOwnedMemory( void );
};

/*
@enum facingActor::facingMode | Type of facing mode the <c facingActor> uses
@emem eightFace | One face for each cardinal direction
@emem twoFace | One face for right and up motion, one for down and left motion
@emem oneFace | One face for all directions (like a regular <c actor>)
@emem fourFace | One face for each of the four primary cardinal directions
*/

/*
@enum facingActor::direction | Direction the <c facingActor> is pointed.
@emem special | No specified direction
@emem southwest | Pointed down and left
@emem south | Pointed down
@emem southeast | Pointed down and right
@emem west | Pointed left
@emem center | Centered (no specified direction)
@emem east | Pointed right
@emem northwest | Pointed up and left
@emem north | pointed straight up
@emem northeast | Pointed up and right
*/

/*
@class An actor which changes its displayed <c animslave> depending on its direction of motion.
@base public | actor
*/
class facingActor :
  public actor {
public:
  enum direction { special, southwest, south, southeast, west, center, east, northwest, north, northeast };
  enum facingMode { eightFace, twoFace, oneFace, fourFace };
  //@cmember Pointers to each of the <c facingActor>'s faces.
  animBitmap * apanmFaces[ NUM_FACES ];
  //@cmember default constructor
  facingActor();
  //@cmember default destructor
  virtual ~facingActor();
  //@cmember Redefined from <mf actor::Advance>
  virtual void Advance();
  //@cmember Turns to a new direction
  virtual void TurnTo( byte bNewDirection );
  //@cmember reassigns the apicnFaces pointers to a new <c directionTable>
  void AssignNewDirectionTable( directionTable * rtMyDirectionTable, facingMode eMode, word wNewIdentity = DEFAULT_ID_ARG );
};

#endif
