#ifndef HACKTIM_H
#define HACKTIM_H

#include <dos.h>
#include <time.h>

//@doc

/*
@class A simple timer class used to check frame rates, etc.  Also includes functions to maintain
a set hack rate (ie maintain a constant screen update rate, etc).
*/
class hackTimer {
private:
  //@access Private Data Members
  //@cmember Clock tick of last hack
  clock_t clkLastHack;
  //@cmember Total time clock has been running, in sec/100
  long lTotalHundredths;
  //@cmember number of hacks made since last reset
  int iNumberOfHacks;
  //@cmember Number of hacks between calculations and resets
  int iHacksPerInterval;
  //@cmember Desired hack length, calculated by <mf hackTimer::SetDesiredHackRate>
  int iDesiredHackLength;
  //@cmember Conversion constant to convert between clock ticks and sec/100
  static float fClocksPerHundredth;

public:
  //@access Public Members
  //@cmember Default constructor
  hackTimer( void );
  //@member ReHack | Resets the clock to zero
  inline void ReHack( void ) { lTotalHundredths = 0; iNumberOfHacks = 0; };
  //@cmember Adds a hack
  void AddHack( void );
  //@cmember Returns the hacks rate in hacks per second over the last interval
  float fHacksPerSecond( void );
  //@cmember Sets the desired hack rate in hacks per second
  void SetDesiredHackRate( int iHacksPerSecond );
  //@cmember Waits until the desired hack length is met
  void WaitForDesiredRate( void );
  //@cmember Sets the fClocksPerHundredth conversion factor
  static void SetClocksPerHundredth( float fNewClocksPerHundredth );
  //@member fGetClocksPerHundredth | Returns current fClocksPerHundredth conversion factor
  inline static float fGetClocksPerHundredth( void ) { return fClocksPerHundredth; };
};

#endif
