#ifndef LISTBOX_H

#define LISTBOX_H

#include "stddefs.h"
#include "yakLib.h"
#include "gadgets.h"
#include "xlib.h"

//@doc

//@enum listBox::direction | Direction of scrolling text
//@emem up | Up
//@emem down | Down

/*
@class A special gadget used to hold a scrolling box of text and allow the user to select
a given line.

@base public | gadget
*/
class listBox : public gadget
{
public:
  //@access Public Members
  enum direction {
    up,
    down
  };
  //@cmember The line number currently at the top of the display
  int iPosition;
  //@cmember The number of displayable lines in the text
  int iNumLines;
  //@cmember The number of displayable lines in the box
  int iBoxLines;
  //@cmember A pointer to the text itself
  char * pcText;
  //@cmember Background color
  color_t colBackground;
  //@cmember Foreground color
  color_t colForeground;
  //@cmember Color of the selected text
  color_t colSelectBackground;
  //@cmember Width of the listBox
  int iWidth;
  //@cmember Height of the listBox
  int iHeight;
  //@cmember Returns a pointer to the given line of text 
  char * pcGetLine( int iLineNumber );
  //@cmember Returns the length of a given line, in characters
  int iLineLength( int iLineNumber );
  //@cmember Draws a line of text to the screen
  void DrawLine( int iLineNumber, screenCoordX_t scx, screenCoordY_t scy, pageHandle_t pagPage );
  //@cmember Draws all the visible text to the screen starting at iLineNumber at a given position
  void DrawText( int iLineNumber, screenCoordX_t scx, screenCoordY_t scy, pageHandle_t pagPage = VisiblePageOffs );
//  //cmember Draws all the visible text to the screen, starting at iLineNumber at the last drawn position
//  void DrawText( int lLineNumber, pageHandle_t pagPage = VisiblePageOffs );
  //@member DrawText | Draws the visible text at the last screen position
  inline void DrawText( pageHandle_t pagPage ) { DrawText( iPosition, scx, scy, pagPage ); };
  //@cmember Scrolls the text one page in the given direction
  void PageMove( direction ePageDirection, pageHandle_t pagPage = VisiblePageOffs );
  //@cmember Scrolls the text one line in the given direction
  void LineMove( direction eLineDirection, pageHandle_t pagPage = VisiblePageOffs );
public:
  char acReturnString[ 80 ];
  listBox(
    screenCoordX_t scxNew,
    screenCoordY_t scyNew,
    char cNewCommandChar,
    int iNewIdentity,
    char * pcNewText,
    color_t colNewForeground,
    color_t colNewBackground,
    color_t colSelectBackground,
    int iNewWidth,
    int iNewHeight,
    gadget::touchType eNewTouchType
  );
  virtual ~listBox();
  virtual void Draw( pageHandle_t pagPage, screenCoordX_t scxNew, screenCoordY_t scyNew );
  virtual gadgetRequest grIsMouseSelected( void ) const;
  virtual void Activate( void );
  virtual word wInterpretKeyStroke( char c );
  virtual word wInterpretMouseClick( void );
  void SelectLine( int iRelativeLine );
  void NewText( char * pcNewText );
  void ParseText( void );
  bFlag bfLoadText( char * pcFilename, yakLib * plib = NULL );
};

#endif
