#ifndef POLY3D_H
#define POLY3D_H

/* DIPOLY.H: Header file for polygon-filling code */
#include "yak3d.h"

/* Describes a series of points (used to store a list of vertices that
   describe a polygon; each vertex is assumed to connect to the two
   adjacent vertices, and the last vertex is assumed to connect to the
   first) */
struct vertexListHeader3d {
   int iLength;                /* # of points */
   struct point2d * pvtx;   /* pointer to list of points */
};

/* Describes the beginning and ending X coordinates of a single
   horizontal line */
struct hLine3d {
   int iXStart; /* X coordinate of leftmost pixel in line */
   int iXEnd;   /* X coordinate of rightmost pixel in line */
   int iY;
   color_t colColor;
   coord3d_t c3ZStart;
   coord3d_t c3ZEnd;
};

/* Describes a Length-long series of horizontal lines, all assumed to
   be on contiguous scan lines starting at YStart and proceeding
   downward (used to describe a scan-converted polygon to the
   low-level hardware-dependent drawing code) */
struct hLineList3d {
   int iLength;                /* # of horizontal lines */
   int iYStart;
   struct hLine3d * phln;   /* pointer to list of horz lines */
};

hLineList3d * phll3dScannedPolygon( vertexListHeader3d * pvtl, int iXOffset, int iYOffset, color_t colColor );

void polygon3d(     /* Draw a convex polygon */
  point2d * pvertices,
  int num_vertices,
  color_t colColor,
  pageHandle_t pagPage
);

/*==================================================================
Now the mighty S-Buffer code...
===================================================================*/

struct sBufferElement {
  hLine3d hl3d;
  sBufferElement * psbelNext;
  sBufferElement * psbelPrev;
};

#define SBUF_FIRSTLINE -32000
#define SBUF_LASTLINE  32000

class sBuffer {
  yakArray<sBufferElement> arrLines;
  sBufferElement sbelHead;
  sBufferElement sbelTail;
  sBufferElement * psbelShuttle;
  int iNextAvailableLine;
public:
  sBuffer( int iInitNumLines );
  void AddLine( const hLine3d &hl3d );
  void AddLineList( const hLineList3d &hll3d );
  void Reset();
  void Draw( pageHandle_t pagPage );
};

void BufferPolygon3d(
  sBuffer & sbuf,
  point2d * pp2d,
  int iNumVertices,
  color_t colColor
);

#endif