#ifndef SCRIPT_H

#define SCRIPT_H

#include <stddef.h>
#include "stddefs.h"
#include "yakList.h"

//@doc

/*
@enum scriptCommand | A command for an actor
@emem doNothing | Do nothing
@emem relativeMove | Move a given number of <t mapCoord_t> units relative to the current position
@emem slide | Move relative to current position, taking a number of steps to do so
@emem turn | Turn to a new direction
@emem animate | Stay in place and animate
*/
/*
@class A command used to move actors on an <c animap>.  It is composed of a command and four
parameters.
@base public | yakObject
*/
class scriptLine : public yakObject
{
  friend class actor;
  friend class facingActor;
public:
  enum scriptCommand {
    doNothing,                              // n/a (4)
    relativeMove,     // deltaX, deltaY, n/a, n/a
    slide,            // deltaX, deltaY, numFrames, n/a
    turn,             // direction, n/a (3)
    animate           // n/a (4)
  };
private:
  //@cmember The actual command contained in this scriptLine
  scriptCommand eCommand;
  //@cmember Parm1
  screenCoord_t scd1;
  //@cmember Parm2
  screenCoord_t scd2;
  //@cmember Parm3
  screenCoord_t scd3;
  //@cmember Parm4
  screenCoord_t scd4;
public:
  //@member scriptLine | Specific constructor
  scriptLine( scriptCommand eNewCommand = doNothing, screenCoord_t scdNew1 = 0,
          screenCoord_t scdNew2 = 0, screenCoord_t scdNew3 = 0, screenCoord_t scdNew4 = 0 );
  //@cmember Copy constructor
  scriptLine( const scriptLine * psclCopyFrom );
  //@member ~scriptLine | Default destructor
  virtual ~scriptLine() {};
};

/*
@class A list of <c scriptLine> objects used to command <c actor> objects.
@base public | pyList
*/
class script : public pyList
{
public:
  //@cmember Default constructor
  script( void );
  //@cmember Copy constructor
  script( const script * pscrCopyFrom );
  //@cmember Default destructor
  virtual ~script();
  //@member psclCurrent | Returns the current line of the script
  inline scriptLine * psclCurrent( void ) { return ( (scriptLine *)( pyHeadData() ) ); };
  //@member AdvanceLine | Advances a line of script
  inline void AdvanceLine() { RemoveHead(); };
  //@member AddLine | Adds a line of script
  void AddLine( scriptLine::scriptCommand eNewCommand, screenCoord_t scdNew1,
          screenCoord_t scdNew2, screenCoord_t scdNew3, screenCoord_t scdNew4 );
  //@member AddCommand | Adds a command to the script, translating it into lines if necessaryy
  void AddCommand( scriptLine::scriptCommand eNewCommand, screenCoord_t scdNew1,
          screenCoord_t scdNew2, screenCoord_t scdNew3, screenCoord_t scdNew4 );
};
#endif
