#ifndef STDDEFS_H

#define STDDEFS_H

#define MAJOR_REVISION 3
#define MINOR_REVISION 0
#define BITSPERNIBBLE 4

//@doc

//@type word | 16-bit unsigned value
//@type byte | 8-bit unsigned value
typedef unsigned short word;
typedef unsigned char byte;

//@type pageHandle_t | Word used to describe offset in video memory of graphics operations
#ifdef __386__
typedef int pageHandle_t;
#else
typedef word pageHandle_t;
#endif

//@type screenCoord_t | Coordinate on the VGA screen
//@type screenCoordX_t | X version of <t screenCoord_t>
//@type screenCoordY_t | Y version of <t screenCoord_t>
#ifdef __386__
typedef int screenCoord_t;
#else
typedef short screenCoord_t;
#endif
typedef screenCoord_t screenCoordX_t;
typedef screenCoord_t screenCoordY_t;

//@struc point on the screen.  Will be used more in future versions.
struct screenPoint {
  screenCoordX_t scx; //@mem x coordinate
  screenCoordY_t scy; //@mem y coordinate
};

//@type mapCoord_t | Generic pixel-wide coordinate on an <c animap>
//@type mapCoordX_t | X version of <t mapCoord_t>
//@type mapCoordY_t | Y version of <t mapCoord_t>
#ifdef __386__
typedef int mapCoord_t;
#else
typedef short mapCoord_t;
#endif
typedef mapCoord_t mapCoordX_t;
typedef mapCoord_t mapCoordY_t;

//@struc point on an animap.  Will be used more in future versions.
struct mapPoint {
  mapCoordX_t mcx; //@mem x coordinate
  mapCoordY_t mcy; //@mem y coordinate
};


//@type color_t | 8-bit value for index into a 256-color palette.
typedef byte color_t;

#define FILENAME_LENGTH 15
//@type filenameString | Array of (FILENAME_LENGTH) characters for filenames.
typedef char filenameString[ FILENAME_LENGTH ];

//@enum bFlag | A boolean flag for success or failure of an operation
enum bFlag {
#ifdef __WATCOMC__
  failure = 0,
  off     = 0,
  false   = 0,
#else
  failure = !( 1 == 1 ),  //@emem Failure of an operation
  off     = !( 1 == 1 ),  //@emem "Off" status of an indicator
  false   = !( 1 == 1 ),  //@emem "False" status of a true-false test
#endif
  success = ( 1 == 1 ),   //@emem Success of an operation
  on      = ( 1 == 1 ),   //@emem "On" status of an indicator
  true    = ( 1 == 1 )    //@emem "True" status of a true-false test.
};

/*
@class Base class for many yakIcons objects.
*/
class yakObject {     
public:
  //@access Public Data Members
  //@cmember Flag showing the temporary/permanent status of the object.
  bFlag bfIsTemporary;
  yakObject( bFlag bfTemp = false ) : bfIsTemporary( bfTemp ) {};
  virtual ~yakObject() {};
};

//Borland C++ doesn't define max and min so you can use templates.
//here they are.
//@func (template) | tMax | Returns the maximum of two input variables
//@tfarg class | T | Class of the compared input variables
//@parm T | x | first variable to compare
//@parm T | y | second variable to compare
template <class T> tMax(T x, T y) {
  return (x > y) ? x : y;
};

//@func (template) | tMin | Returns the minimum of two input variables
//@tfarg class | T | Class of the two compared input variables
//@parm T | x | first variable to compare
//@parm T | y | second variable to compare
template <class T> tMin(T x, T y) {
  return (x < y) ? x : y;
};

//the loop and exitWhen macros are used to implement "loop-with-exit"
//constructions
#define loop for(;;)
#define exitWhen(flag) if (flag) break

//SafelyFree() deletes the contents of a pointer using delete if it contains
//a non-null value, then sets the value equal to NULL.
#define SafelyFree(p) if ( p != NULL ) { delete p; } p = NULL
//SafelyFreeArray() deletes the contents of a pointer using delete[] if it contains
//a non-null value, then sets the value equal to NULL.
#define SafelyFreeArray(p) if ( p != NULL ) { delete[] p; } p = NULL

void SaveMode( void );
int iBound( int i, int iMinLimit, int iMaxLimit );
int iNextMultiple( int i, int iBase );

/*
@func crops the number i to within the limit specified, "rolling
over" the side.  For example, iRollover( 7, 5 ) = 2.
@rdesc int, the "rolled over" number.
*/
inline int iRollover(
  int i,                //@parm number to be modified
  int iLimit            //@parm limit to the number
)
{
  int iTemp = i % iLimit;
  iTemp += (iTemp < 0) ? iLimit : 0;
  return iTemp;
}

#include "debug.h"


/*
@func checks to see if a given integer is in a given range.
Bounds are inclusive, ie bfIsInRange( 2, 2, 8 ) == true.
@rdesc one of the following:
@flag success | if the number is in the given range
@flag failure | if it is not
*/
inline bFlag bfIsInRange(
  int i,           //@parm number to check
  int iMinLimit,   //@parm min "in range" limit
  int iMaxLimit    //@parm max "in range" limit
)
{
  //make sure the limits are valid
  ASSERT( iMinLimit <= iMaxLimit );

  //now check our logic.
  if ( ( iMinLimit <= i ) &&
     ( i <= iMaxLimit ) ) {
    return true;
  }
  else {
    return false;
  }
}



#endif
