#ifndef YAK3D_H
#define YAK3D_H

#include "yakList.h"
#include "yakArray.h"
#include "yaklib.h"
#include "animbitm.h"
//@doc




typedef double coord3d_t;

struct point2d {
  int iX;
  int iY;
  coord3d_t c3Z;  //used to sort points
};

struct point3d {
  coord3d_t c3X;
  coord3d_t c3Y;
  coord3d_t c3Z;
  coord3d_t c3W;
  point3d operator- ( const point3d p3dSubtrahend ) const;
  point3d operator+ ( const point3d p3dAddend ) const;
  point3d operator+= ( const point3d p3dAddend );
  point3d p3dCrossProduct( const point3d p3dOperand ) const;
  double dMagnitude( void ) const;
  void Normalize( void );
  double dDotProduct( const point3d p3d ) const;
};

typedef point3d vector3;


typedef double angle_t;


enum axis_t {
  xAxis,
  yAxis,
  zAxis,
  special
};

class threeTransform {
public:
	//mfunc default constructor creates an identity transform
  threeTransform( void );
  double adEntries[ 4 ][ 4 ];
  void TransformPoint3d( point3d * pp3dIn, point3d * pp3dOut ) const;
  threeTransform operator+( const threeTransform xfmAddend ) const;
  void Rotate( axis_t axsRotateAround, angle_t angDelta );
  void Translate( const vector3 vc3Delta );
  void TranslateTo( const point3d p3dNew );
};

/*==================================================================
code for projecting 3d to 2d
===================================================================*/

class viewport {
  double dProjectionRatio;
  double dWidth;
  double dHeight;
  int iLeft;
  int iTop;
  int iRight;
  int iBottom;
public :
  void Set( int iNewLeft, int iNewTop, int iNewRight, int iNewBottom, double dNewProjectionRatio );
	void ProjectPoint3d( point3d * pp3dIn, point2d * pp2dOut ) const;
  void ScaleDistance( int iLength, coord3d_t c3Z, int * piScaledLength ) const;
};


/*==================================================================
3-d object color table; used to simulate solid-color ranges in a
yakpalette
===================================================================*/

struct colorTable3dEntry {
  int iNumColors;
  int iPaletteIndex;
};

class colorTable3d {
  yakArray<colorTable3dEntry> arrEntries;
public:
	colorTable3d( int iNewNumEntries );
  void SetEntry( int iNumEntry, int iNumColors, int iPaletteEntry );
  color_t colGetColor( int iColor, double dPercent = 1.0 ) const;
  static vector3 vc3Light;
  static colorTable3d * pctbCurrent;
};


/*==================================================================
Basic 3-d drawing element
===================================================================*/
class sBuffer;


class element3d {
public :
	coord3d_t c3Z;
  virtual void Draw( pageHandle_t pagPage ) = 0;
  virtual void DumpToSBuffer( sBuffer & sbuf ) = 0;
};


/*==================================================================
Basic 3-d object
===================================================================*/

class world3d;
/*
@class The basic building block of yakIcons 3-d, the <c object3d> class
is a base class for all 3-d objects in yakIcons.
*/
class object3d : public yakObject {
friend world3d;
protected :
  threeTransform xfmToWorld;
  world3d * pw3dOwning;
public :
  point3d p3dMidpoint;
  angle_t aangOrientation[ 3 ];
  virtual void Draw( pageHandle_t pagPage ) = 0;
  virtual void TransformToView( const viewport & vpt ) = 0;
  void Rotate( axis_t axsRotateAround, angle_t angDelta ) { xfmToWorld.Rotate( axsRotateAround, angDelta ); };
  void Translate( vector3 vc3Delta ) { p3dMidpoint += vc3Delta; xfmToWorld.Translate( vc3Delta ); };
  void TranslateTo( point3d p3dNew ) { p3dMidpoint = p3dNew; xfmToWorld.TranslateTo( p3dNew ); };
  void PutToWorld( world3d * pw3d );
  virtual void DumpElements( element3d *** pppelBuffer, int * piNumElements ) {	};
  virtual void DumpElementsToSBuffer( sBuffer &sbuf ) {	};
//  void RemoveFromWorld( void );
};



/*
@class a 3-d object composed of flat polygons
*/
class polyhedron : public object3d {
  int iNumVertices;
  //now several arrays of vertices for all stages of the 3d-to-2d process
  point3d * pp3dObjectSpace;
  point3d * pp3dViewSpace;
  point2d * pp2dScreenSpace;
  //@mdata number of faces
  enum _faceColorType {
    normal = 1,		//just use the color directly
    shaded = 2    //interpret the color from the current colorTable3d
  };
  class _face : public element3d {
    friend class polyhedron;
    int iNumVertices;
    yakArray<int> arrVertices;
    yakArray<point2d> arrScreenPoints;
    point2d * pp2dVertexHeap;
    enum _faceColorType {
      normal = 1,		//just use the color directly
      shaded = 2    //interpret the color from the current colorTable3d
    };
    _faceColorType eColorType;
    color_t colColor;
    color_t colDisplayColor;
    double dReflectivity;
    point3d p3dNormalObject;
    point3d p3dNormalView;
    vector3 v3dNormal;
    void Draw( pageHandle_t pagPage );
    void DumpToSBuffer( sBuffer & sbuf );
    void CalculateColor( void );
    bFlag bfIsBackface( void );
  };
  yakArray<_face> arrFaces;
public :
  polyhedron( void );
  ~polyhedron( void );
  void SetNumbers( int iNewNumVertices, int iNewNumFaces );
  void SetVertex( int iVertex, coord3d_t c3X, coord3d_t c3Y, coord3d_t c3Z );
  void SetFace( int iNumFace, int iNumFaceVertices, int * piVertexNumbers );
  void SetFaceColor( int iNumFace, _faceColorType eNewColorType, color_t colColor, double dReflectivity );
  void DrawWireFrame( pageHandle_t pagPage, color_t colColor ) const;
  void Draw( pageHandle_t pagPage );
  void TransformToView( const viewport &vpt );
  virtual void DumpElements( element3d *** pppelBuffer, int * piNumElements );
  virtual void DumpElementsToSBuffer( sBuffer & sBuf );
  bFlag bfLoad( const char * CszFilename, yakLib * plib = NULL );
};


class icon3d : public object3d {
  int iWidthObject;
  int iHeightObject;
  class _3dIcon : public animslave, public element3d {
    public:
    icon3d * pi3dOwning;
    int iWidthView;
    int iHeightView;
	  point2d p2dScreenCenter;
    virtual void Draw( pageHandle_t pagPage );
    virtual void DumpToSBuffer( sBuffer & sbuf );
  };
public :
  _3dIcon myElement;
  void Draw( pageHandle_t pagPage );
  void TransformToView( const viewport &vpt );
  void SetDimensions( int iNewWidth, int iNewHeight ) { iWidthObject = iNewWidth; iHeightObject = iNewHeight; };
  virtual void DumpElements( element3d *** pppelBuffer, int * piNumElements );
};

/*==================================================================
3dWorld class definition
===================================================================*/

class world3d {
friend object3d;
  pyList lstObjects;
public :
  void Draw( pageHandle_t pagPage );
  void DumpToSBuffer( sBuffer & sbuf );
  void TransformToView( const viewport &vpt );
};


#endif