#ifndef YAKANGLE_H
#define YAKANGLE_H

#include "yakArray.h"

/*==================================================================
A yakAngle is simply a fixed-point angle with 8.8 precision, where
0yAng = 256.0yAng = 0 degrees = 360 degrees
===================================================================*/

class yakAngle {
  static int iPrecisionShift;
  static yakArray<float> arrfCosineTable;
  static yakArray<float> arrfSineTable;
public:
  word wValue;
  inline yakAngle( word wNewValue = 0 ) : wValue( wNewValue ) {};
  inline static float fCosine( yakAngle yAng ) {
    return arrfCosineTable[ yAng.wValue >> iPrecisionShift ];
        };
  inline static float fSine( yakAngle yAng ) {
    return arrfSineTable[ yAng.wValue >> iPrecisionShift ];
        };
  static void InitSineTables( int iNewPrecision = 8 );
  inline yakAngle operator+( const int i ) const {
    return yakAngle( wValue + ( i << 8 ) );
  };
  inline yakAngle operator+( const yakAngle yang ) const {
    yakAngle yangTemp;
    yangTemp.wValue = yang.wValue + wValue;
    return yangTemp;
  };
  inline yakAngle operator+=( const yakAngle yang ) {
    wValue += yang.wValue;
    return *this;
  }
  inline yakAngle operator-( const yakAngle & yAng ) const {
    return yakAngle( wValue + yAng.wValue );
  };
  inline yakAngle operator-( const int i ) const {
    return yakAngle( wValue - ( i << 8 ) );
  }
  inline yakAngle operator=( const int i ) {
    wValue = i << 8;
    return *this;
  }
  inline yakAngle operator*( const float f ) {
    return yakAngle( ( word )( ( float )wValue * f ) );
  }
};

#endif
