#ifndef __386__
#ifndef YAKEMS_H
#define YAKEMS_H

#include "yaklist.h"
#include <dos.h>

//@doc

//the following are assembler routines to allocate, map, and free
//pages
extern "C" {
extern int iEms_err;
extern word cdecl wEms_verify( void );
extern word cdecl wEms_alloc( word wPageCount );
extern word cdecl wEms_map( word wPhysicalPage, word wHandle, word wLogicalPage );
extern word cdecl wEms_free( word wHandle );
extern word cdecl wEms_getframe( void );
}

class emsBlock;
/*
@class A generic "chunk of data" class which can hold its data in EMS memory.
@base public | yakObject
*/
class emsData : public yakObject
{
  friend class emsBlock;
private:
  //@access Private Members
  //@cmember Pointer to the owned data
  byte * pbDataPtr;
  //@cmember Handle of the EMS page which owns this data
  int iHandle;
  //@cmember Size of this data block
  word wSize;
public:
  //@access Public Members
  //@member iGetHandle | Returns iHandle
  inline int iGetHandle( void ) const { return iHandle; };
  //@member wGetSize | Returns wSize
  inline word wGetSize( void ) const { return wSize; };
  //@cmember Default constructor
  emsData( void );
  //@cmember Default destructor
  virtual ~emsData( void );
  //@cmember Returns a usable pointer to the data
  byte * pbData( void ) const;
  //@cmember Allocates and copies data from another memory location
  byte * pbUseData(void * pvDataToUse, word wSizeMoved);
  //@cmember Allocates a given section of memory
  byte * pbAllocate(word wSizeToAllocate);
  //@cmember DeAllocates allocated memory
  void DeAllocate( void );
};

/*
@class A 64k-large chunk of EMS memory which is partitioned into many discrete emsData
objects.
@base public | yakObject
*/
class emsBlock : public yakObject
{
  friend class emsData;
#define PAGES_IN_A_BLOCK 4
private:
  //@access Private Members
  //@cmember EMS page handle for this block
  int iHandle;
  //@cmember List of all <c emsData> objects residing in this block
  pyList LDataList;
  //@cmember EMS page frame address
  static word wPageFrame;
  //@cmember Static list of allocated emsBlock units
  static pyList LBlockList;
  //@cmember Handle of EMS page currently mapped into the page frame
  static int iCurrentHandle;      
public:
  //@access Public Members
  //@cmember default constructor
  emsBlock( void );
  //@cmember default destructor
  virtual ~emsBlock( void );
  //@cmember Swaps this emsData block into the EMS page frame
  void SwapIn( void ) const;
  //@cmember Defragments this emsBlock
  void Defragment( void );
  //@cmember Initializes yakIcons EMS services
  static bFlag bfInit( void );
  //@cmember Defragments all allocated emsBlock objects
  static void DefragmentAllEMS( void );
  //@cmember Frees all allocated EMS, deAllocating all <c emsData> objects in EMS
  static void FreeAllEMS( void );
  //@cmember Shows allocation status of EMS memory
  static void ShowBlockStatus( void );
};

#endif
#endif //__386__
