#ifndef YAKFONT_H
#define YAKFONT_H

#include "stddefs.h"
#include "yakLib.h"
#include "yakwin.h"

//@doc

/*
@class A font class which holds monochrome fonts of small size (less than 8 pixels wide)
*/
class yakFont
{
private:
  //@access Private Members
  //@cmember The current active font
  static const yakFont * pfntCurrent; 
public:
  //@access Public Members
  //@cmember Ascii code of the first character in the font
  word wStartChar;
  //@cmember Height of characters in pixels
  byte bCharHeight;
  //@cmember Width of characters in pixels
  byte bCharWidth;
  //@cmember Number of characters in the font
  byte bNumberOfChars;
  //@cmember Data for the font, including the first three items (ie character data
  //starts at fontData[4])
  byte * pbFontData;
  //@cmember Default destructor
  ~yakFont( void );
  //@cmember Default constructor
  yakFont( void );
  //@cmember Specific constructor
  yakFont( int iStartChar, int iCharWidth, int iCharHeight, int iNumChars );
  //@cmember Specific constructor (loads from disk)
  yakFont( const char * CszFilename, yakLib * plib = NULL );
  //@cmember Loads font from disk
  bFlag bfLoad( const char * CszFilename, yakLib * plib = NULL );
  //@cmember Saves font to disk
  bFlag bfSave( const char * CszFilename ) const;
  //@cmember Sets this as the active font
  void Use( void ) const;
  //@member Printf | Prints a string to the screen
  void Printf( pageHandle_t pagPage, screenCoordX_t scx, screenCoordY_t scy,
    color_t colColor, const char * pcLn, ... ) const;
  //@member BgPrintf | Prints a string to screen with a solid background color
  void BgPrintf( pageHandle_t pagPage, screenCoordX_t scx, screenCoordY_t scy,
    color_t colForeground, color_t colBackground, const char * pcLn, ... ) const;
  //@member OutlinePrintf | Prints a string to screen with a background color around each
  //letter
  void OutlinePrintf( pageHandle_t pagPage, screenCoordX_t scx, screenCoordY_t scy,
    color_t colForeground, color_t colBackground, const char * pcText, ... ) const;
  //@member pfntGetCurrent | Returns pfntCurrent
  inline static const yakFont * pfntGetCurrent( void ) { return pfntCurrent; };
  //@member iCurrentWidth | Returns the width of the current active font
  inline static int iCurrentWidth( void ) { ASSERT( pfntCurrent ); return pfntCurrent->bCharWidth; };
  //@member iCurrentHeight | Returns the height of the current active font
  inline static int iCurrentHeight( void ) { ASSERT( pfntCurrent ); return pfntCurrent->bCharHeight; };
};


#endif

