#ifndef YAKKEYS_H
#define YAKKEYS_H

#include "stddefs.h"
//header file for yakKeyboard; adapted from STK code by Jari Karjala
const int CnKeys = 128;

//@doc

//@enum yakKeyboard::keyLabel | Labels for the keys

//@class A keyboard class which replaces the default keyboard interrupt handler with
//a handler that can accept multiple keypresses.
class yakKeyboard
{
private:
#ifdef __WATCOMC__
  static void __interrupt __far ( *pvfunOldHandler )();
  static void __interrupt __far NewHandler();
#else
  static void interrupt ( *pvfunOldHandler )( ... ); //pointer to
    //the old keyboard interrupt handler
  static void interrupt NewHandler( ... );  //the new keyboard interrupt
    //handler that we're installing
#endif
  //@cmember Array of bytes holding the keypress status
  static byte abKeys[ CnKeys ]; 
  //@cmember Holds the status of the keyboard handler, installed or not.
  static bFlag bfIsInstalled;
public:
  //codes for the keys follow
  enum keyLabel {escape = 1, num1, num2, num3, num4, num5, num6, num7,
     num8, num9, num0, underscore, equals, backSpace, tab,
     charQ, charW, charE, charR, charT, charY, charU, charI,
     charO, charP, openBraces, closeBraces, enter, ctrl, charA,
     charS, charD, charF, charG, charH, charJ, charK, charL,
     colon, quote, tilde, leftShift, bkSlash, charZ, charX,
     charC, charV, charB, charN, charM, comma, dot, fdSlash,
     rightShift, KPAsterisk, alt, space, capsLock, f1, f2, f3, f4,
     f5, f6, f7, f8, f9, f10, numLock, scrollLock, home, KP7 = 71,
     upArrow, KP8 = 72, pageUp, KP9 = 73, KPminus, leftArrow,
     KP4 = 75, KP5, rightArrow, KP6 = 77, KPPlus, end, KP1 = 79,
     downArrow, KP2 = 80, pageDn, KP3 = 81, insert, KP0 = 82, del,
     KPdot = 83, f11=89, f12};
  //@cmember Checks to see if a given key is pressed
  static bFlag bfIsPressed( keyLabel key );
  //@cmember Checks to see if any key is pressed, and returns that key
  static keyLabel keyPressed( void );
  //@cmember Waits for a keypress
  static keyLabel keyGet( void );
  //@cmember Installs the keyboard handler
  static void Install( void );
  //@cmember Removes the keyboard handler
  static void Remove( void );
  //@cmember Returns the value of bfIsInstalled
  static bFlag bfGetIsInstalled( void ) { return bfIsInstalled; };
};

#ifndef YAKKEYSUNIT
extern yakKeyboard keyboard;  //keyboard automatically included
#endif
#endif
