#ifndef YAKMOUSE_H

#define YAKMOUSE_H

#include "stddefs.h"
#include "xlib.h"
#include "xmouse.h"

//@doc

/*
@enum yakMouse::buttonType | Type of mouse button check to make
@emem noButtons | No buttons are pressed
@emem leftButton | The left button is pressed
@emem rightButton | The right button is pressed
@emem reset | Resets the check for mouse clicks
@emem eitherButton | Either button is pressed
*/

//@class A generic mouse object
class yakMouse
{
private:
  //@access Private Members
  //@cmember Number of "levels" the mouse is hidden
  static int iMouseHideLevel;
public:
  //@access Public Members
  enum buttonType {
    noButtons    = 0,
    leftButton   = 0x01,
    rightButton  = 0x02,
    reset        = 0xff,
    eitherButton = 0x03
  };
  //@cmember Initializes mouse handler
  void Init( void );
  //@member Remove | Removes the mouse handler
  inline void Remove( void ) const { x_mouse_remove(); };
  //@cmember Enables showing of the mouse
  void Show( void );
  //@cmember Hides the mouse from view
  void Hide( void );
  //@cmember Sets the mouse image to the standard yakIcons mouse image
  void BeStandardMouse( void ) const;
  //@member SetImage | Redefines the mouse image
  inline void SetImage( char * pcMouseDef, color_t colColor ) const { x_define_mouse_cursor( pcMouseDef, colColor ); };
  //@member Update | Updates the mouse position
  inline void Update( void ) const { x_update_mouse(); };
  //@member Reposition | Repositions the mouse
  inline void Reposition( screenCoordX_t scxNew, screenCoordY_t scyNew ) const { x_position_mouse( scxNew, scyNew ); };
  //@member Freeze | Freezes the mouse
  inline void Freeze( void ) const { MouseFrozen = true; };
  //@member Release | Releases the mouse from a frozen condition
  inline void Release( void ) const { MouseFrozen = false; };
  //@member Confine | Confines the mouse to a bounding box
  inline void Confine( screenCoordX_t scxLeft, screenCoordY_t scyTop,
    screenCoordX_t scxRight, screenCoordY_t scyBottom ) const { x_mouse_window( scxLeft, scyTop, scxRight, scyBottom ); };
  //@member wX | Returns the X position of the mouse
  inline word wX( void ) const { return (word)MouseX; };
  //@member wY | Returns the Y position of the mouse
  inline word wY( void ) const { return (word)MouseY; };
  //@member Display | Displays the mouse cursor
  inline void Display( pageHandle_t pagPage, screenCoordX_t scx, screenCoordY_t scy,
    screenCoordY_t scyTopClip, screenCoordY_t scyBotClip ) const { x_put_cursor( scx, scy, scyTopClip, scyBotClip, pagPage ); };
  //@member wButtonStatus | Returns the status of the mouse buttons
  inline word wButtonStatus( void ) const { return (word)MouseButtonStatus; };
  //@member bfIsPressed | Checks to see if a given mouse button is pressed
  inline bFlag bfIsPressed( buttonType eButtons ) const { return ( MouseButtonStatus & eButtons ) ? success : failure; };
  //@cmember Checks to see if a given mouse button is clicked
  bFlag bfIsClicked( buttonType eButtons ) const;
  //@cmember Checks to see if the mouse is in a given bounding box
  bFlag bfIsInBox( screenCoordX_t scxLeft, screenCoordY_t scyTop, screenCoordX_t scxRight, screenCoordY_t scyBottom ) const;
};

#ifndef YAKMOUSE_CPP
extern yakMouse mouse;
#endif

#endif
