#ifndef YAKPAL_H
#define YAKPAL_H

#include "stddefs.h"
#include "yaklib.h"
#include "xlib.h"
#include "xpal.h"

#define NUM_COLORS 256
#define NUM_COMPONENTS 3
//@doc

//@type colorComponent_t | A byte holding the red, green, or blue component of a 24-bit color.
typedef byte colorComponent_t;

class yakPaletteMaintenance;
/*
@enum yakPalette::rotateDirection | Directions to use for palette "rotation"
@emem down | Down
@emem up | Up
*/

/*
@class A 256-color palette holding three 8-bit <t colorComponent_t>s for each color entry.
*/
class yakPalette
{
friend class yakPaletteMaintenance;
  //@access Private Members
  //@cmember Holds all the <t colorComponent_t> data for the palette
  colorComponent_t acmpPaletteData[ NUM_COLORS * NUM_COMPONENTS ];
public:
  //@access Public Members
  enum rotateDirection { down, up };
  //@member yakPalette | Default constructor
  yakPalette( void ) {};
  //@member yakPalette | Specific constructor, loads from disk
  yakPalette( const char * CszFilename, yakLib * plib = NULL ) { bfLoad( CszFilename, plib ); };
  //@cmember Loads palette from disk
  bFlag bfLoad( const char * CszFilename, yakLib * plib = NULL );
  //@cmember Saves palette to disk
  bFlag bfSave( const char * CszFilename ) const;
  //@member Get | Gets palette data from the video card
  void Get( word wStartColor = 0, word wNumColors = NUM_COLORS ) { x_get_pal_raw( acmpPaletteData, wNumColors, wStartColor ); };
  //@member Put | Puts palette data to the video card
  void Put( word wStartColor = 0, word wNumColors = NUM_COLORS ) { x_put_pal_raw( acmpPaletteData, (word)wNumColors, (word)wStartColor ); };
  //@cmember Rotates entries in a yakPalette
  void Rotate( rotateDirection eMyDirection, word wStartColor = 0, word wNumColors = NUM_COLORS );
  //@cmember Sets the <t colorComponent_t> colors of a given <t color_t> entry
  void SetColor( color_t colColor, colorComponent_t cmpRed, colorComponent_t cmpGreen, colorComponent_t cmpBlue );
  //@cmember Fades the colors in a yakPalette by a given intensity
  void Fade( char cIntensity, word wStartColor = 0, word wNumColors = NUM_COLORS );
  //@member SetToTween | Sets the palette colors to values between two other palettes
  void SetToTween( yakPalette * ppalInitial, yakPalette * ppalFinal,
    int iStepNumber, int iNumberOfSteps,
    word wStartColor = 0, word wNumColors = NUM_COLORS );
};
#endif
