#ifndef YAKSTICK_H
#define YAKSTICK_H

#include "stddefs.h"

//@doc

/*
@enum yakStick::buttonType | Joystick button ID
@emem button1 | Button 1
@emem button2 | Button 2
*/
/*
@class A joystick class object that can read joystick 1 or 2.  It features automatic
calibration and consistent value reporting from 0 to 1.  Two joystick objects
( joystick1 and joystick2 ) are automatically created for the user; creating
more would be counterproductive, so use the premade objects rather than creating
your own.  The routine finds the "center" of the sticks using <mf yakStick::SetCenter>
and extrapolates the min and max bounds of the joystick.  It expands these bounds
when values exceeding them are encountered.  To approximate the non-linear curve
of a charging joystick, the routine uses a straight-line approximation from min
value to center and another straight-line approximation from center to max value.
If the center-setting function does not work, the raw values for minimum, center,
and maximum on each axis can be manually set.
*/
class yakStick
{
public:
  //@access Public Members
  enum buttonType {
    button1,
    button2
  };
  //@cmember Stick Number
  byte bStickNumber;
  //@cmember Number of this joystick (1 or 2)
  yakStick( byte bNumber );
  //@cmember X position of joystick at last poll
  float fLastX( void ) const;
  //@cmember Y position of joystic at last poll
  float fLastY( void ) const;
  //@cmember Checks to see if the given button was pressed at last poll
  bFlag bfWasLastPressed( buttonType eButtons ) const;
  //@cmember Polls joysticks to get new stick data
  static void Poll( void );
  //@cmember sets max number of polls to make to the joystick
  static void SetMaxPolls( int iNewMaxPolls );
  //@cmember gets the current max # of polls to make to the joystick
  static int iGetMaxPolls( void );
  //@cmember Resets joystick's autocalibration values
  void Reset( void );
  //@cmember Sets the joystick's autocalibration values for the center position
  void SetCenter( void );
  //@cmember Last raw x position encountered
  int iLastX;
  //@cmember Minimum raw X position encountered
  int iMinX;
  //@cmember raw X position of the "center"
  int iMiddleX;
  //@cmember Maximum raw X position encountered
  int iMaxX;

  //@cmember Last raw Y position encountered
  int iLastY;
  //@cmember Minimum raw Y position encountered
  int iMinY;
  //@cmember raw Y position of the "center"
  int iMiddleY;
  //@cmember Maximum raw Y position encountered
  int iMaxY;
private:
  //@access private members
  //@cmember Last button status
  int iLastButtons;
  //@cmember adjusts max and min values to incorporate new values
  void SetMinMax( void );
  //@cmember holds the number of max iterations to make when polling the joystick
  static int iMaxPolls;
};

#ifndef YAKSTICK_CPP
extern yakStick joystick1;
extern yakStick joystick2;
#endif

#endif
