//contains header information for the yakWindows classes; at this
//stage, just yakWindowPane
#ifndef YAKWIN_H
#define YAKWIN_H

#include "stddefs.h"
#include "yaklist.h"
#include "xlib.h"
#include "gadgets.h"
#include <stdlib.h>
//@doc

#define BOTHPAGES 65530u
/*
@class A yakWindowPane is basically an area of the screen that can save itself to memory
and then restore itself.  Although not a yakWindowPane itself, the mouse is a good example
of this (it stores the area underneath it and then restores it when it moves)
*/
class yakWindowPane
{
protected:
  //@access Protected Members
  //@cmember First chunk of graphic data saved
  byte * pbGraphicData;
  //@cmember Second chunk of graphic data saved
  byte * pbGraphicData2;
  //@cmember Frees up saved graphic data
  void FreeGraphicData( void );
public:
  //@access Public Members
  //@cmember X coordinate of top left corner
  screenCoordX_t scx;
  //@cmember Y coordinate of top left corner
  screenCoordY_t scy;
  //@cmember Width of window in pixels
  int iWidth;
  //@cmember Width of window in nibbles (iWidth / 4)
  int iNibbleWidth;
  //@cmember Height of window in pixels
  int iHeight;
  //@cmember Default constructor
  yakWindowPane( void );
  //@cmember Specific constructor
  yakWindowPane( screenCoordX_t scxLeft, screenCoordY_t scyTop, screenCoordX_t scxRight, screenCoordY_t scyBottom);
  //@cmember Default destructor
  virtual ~yakWindowPane( void );
  //@cmember Saves screen data at the given location and page to memory
  void Save( screenCoordX_t scxLeft, screenCoordY_t scyTop, pageHandle_t pagPage = HiddenPageOffs );
  //@cmember Saves screen data at ( scx, scy ) and the given page to memory
  void Save( pageHandle_t pagPage = BOTHPAGES );
  //@cmember Restores saved data to the screen at the given page
  void Restore( pageHandle_t pagPage ) const;
  //@cmember Restores saved data to the screen at all applicable pages
  void Restore( void ) const;
};

/*
@enum yakWindow::flags | Flags holding window options
@emem isSizeable | Whether or not the window can be resized
@emem isDraggable | Whether or not the window can be dragged about
@emem isCloseable | Whether or not the window can be closed
*/

/*
@class A movable, tileable window class for use in GUI applications.
@base public | yakWindowPane
@base public | yakObject
*/
class yakWindow :
  public yakWindowPane,
  public yakObject {
#define DEFAULT_BOXCOLOR 0
#define DEFAULT_TEXTCOLOR 15
#define DEFAULT_TITLECOLOR 63
#define BORDERCOLOR 10
#define BORDERWIDTH 2
#define HIGHLIGHTCOLOR 15
#define LOWLIGHTCOLOR 5
#define HIGHLIGHTWIDTH 1
#define INACTIVE_COLOR_CHANGE -2
#define SELECTORWIDTH 5
private:
protected:
  //@access Protected Members
  //@cmember Window to kill after next advance
  static yakWindow * pwndKill;
  //@cmember The current "stack" of windows
  static pyList pylWindowStack;
  //@cmember ID number of this window
  word wWindowNumber;
  //@cmember Bflag holding current "open or not" status
  bFlag bfIsOpen;
  //@cmember Drags the window with the mouse
  void Drag( void );
  //@cmember Resizes the window with the mouse
  void Size( void );
  //@cmember Shuffles this window to the top of the stack
  void ShuffleStack( pageHandle_t pagPage, bFlag bfSave );
  //@cmember Draws the stack of windows, showing this one on top
  static void DrawStack( pageHandle_t pagPage, bFlag bfSave );
public:
  enum flags {
    isSizeable = 0x01,
    isDraggable = 0x02,
    isCloseable = 0x04,
  };
  //@cmember Window interaction options
  flags eFlags;
  //@cmember Color of the title bar background
  color_t colTitleColor;
  //@cmember Color of the title bar text
  color_t colTextColor;
  //@cmember Color of the window background
  color_t colBoxColor;
  //@cmember Title of the window
  char acTitle[ 80 ];
  //@cmember Current active on-top window
  static yakWindow * pwndActive;
  //@cmember Default constructor
  yakWindow( void );
  //@cmember Specific constructor
  yakWindow( screenCoordX_t scxLeft, screenCoordY_t scyTop, screenCoordX_t scxRight, screenCoordY_t scyBottom );
  //@cmember Default destructor
  virtual ~yakWindow( void );
  //@cmember Opens the window and puts it to the top of the stack
  virtual void Open( void );
  //@cmember Closes the window and activates the next window down
  virtual void Close( void );
  //@cmember Shows the window as activated to all applicable pages
  virtual void ShowActivated( void ) const;
  //@cmember Shows the window as activated to a given page
  virtual void ShowActivated( pageHandle_t pagPage ) const;
  //@cmember Shows the window as deactivated to all applicable pages
  virtual void ShowDeActivated( void ) const;
  //@cmember Shows the window as deactivated to a given page
  virtual void ShowDeActivated( pageHandle_t pagPage ) const;
  //@cmember Shuffles this window to the top of the stack
  virtual void ShuffleToTop( void );
  //@cmember Draws the window to a given page (this is the one to redefine for
  //user-defined inheriting yakWindow classes)
  virtual void Draw( pageHandle_t pagPage );
  //@cmember Draws the window to all applicable pages
  virtual void Draw( void );
  //@cmember Interprets the mouse status and its affect on the window
  virtual gadgetID_t gidInterpretMouseClick( void );
  //@cmember Interprets a given character as a window command
  virtual gadgetID_t gidInterpretKeyStroke( char c );
  //@cmember Checks to see if this window is selected
  virtual bFlag bfIsSelected( void ) const;
  //@cmember Checks to see if the window is being selected for dragging
  virtual bFlag bfIsDragSelected( void ) const;
  //@cmember Checks to see if the window is being selected for sizing
  virtual bFlag bfIsSizeSelected( void ) const;
  //@cmember Checks to see if the window is being selected for closing
  virtual bFlag bfIsCloseSelected( void ) const;
  //@cmember Draws all windows in the stack from the bottom up
  static void DrawAll( pageHandle_t pagPage = (word)BOTHPAGES );
  //@cmember Closes all windows from the top down
  static void CloseAllWindows( void );
  //@cmember Selects this window to the top of the stack
  static void SelectToTop( void );
  //@cmember Returns the number of windows in the stack
  static int iNumWindows( void );
  //@cmember Checks each window for its current status and activates them as necessary
  static gadgetID_t gidAdvance( void );
};

#endif
