#include "yakicons.h"
#include <conio.h>

/*==================================================================
Demorts.cpp contains simple functions common to all the demonstration
programs.
===================================================================*/

yakLib libDemo;
yakFont fntDemo;
yakPalette palDemo;

/*
SetUpGraphics() sets up a simple 320x200 graphics mode page.
*/
const screenCoordX_t CscxVirtualScreenWidth     = 400;
const screenCoordY_t CscyVirtualScreenHeight    = 240;
const screenCoordY_t CscyInitialScanHeight              = 170;
const char * CszLibName = "demos";
const char * CszFntName = "6x8b.fnt";
const char * CszPalName = "standard.ypl";

/*
InitDemoObjects() opens the library and loads the font and palette
*/
void InitDemoObjects(
  void
)
{
  //now set up our structures: open the library
  bFlag bfOK = true;
  bfOK = libDemo.bfOpen( CszLibName );
  ASSERT( bfOK );
  //load the font
  bfOK = fntDemo.bfLoad( CszFntName, &libDemo );
  ASSERT( bfOK );
  //then load the palette and use it.
  bfOK = palDemo.bfLoad( CszPalName, &libDemo );
  ASSERT( bfOK );
}


void SetUpGraphics(
  void
)
{
	//set up 320x200 x mode with a 360 pixel wide virtual screen
	x_set_mode( X_MODE_320x240, CscxVirtualScreenWidth );
  //set up a splitscreen
  x_set_splitscreen( CscyInitialScanHeight );
  //set up double-buffering with given virtual screen height
  //( must do this AFTER setting up the split screen )
  x_set_doublebuffer( CscyVirtualScreenHeight );
  //hide the split screen until we need it.
  x_hide_splitscreen();
  //...and initialize text routines.
  x_text_init();
  //now use our loaded objects
  fntDemo.Use();
  palDemo.Put();
}


/*
QuitFromGraphics() goes back to text mode.
*/
void QuitFromGraphics(
  void
)
{
  x_text_mode();
}


/*
SpewText() displays a text file loaded from libDemo and waits for a keypress
to continue
*/
void SpewText(
  const char * CszFilename
)
{
  byte * pb = NULL;
  long lSize = 0;
  libDemo.bfLoadSize( CszFilename, &pb, &lSize );
  byte * pbCount = pb;
  while ( lSize-- ) {
    cout << *pbCount++;
  }
  cout << "\n\nPress a key to continue.\n";
  getch();
  SafelyFree( pb );
}
