#include "yakicons.h"
#include "demorts.h"
#include <conio.h>
#include <stdlib.h>

/*==================================================================
icondemo is a simple demonstration of the yakBitmap and animBitmap classes.
It displays the Winged Yak logo static yakBitmap in normal, masked,
collision, and zoomed modes, then shows the different stages of
grumjug animation in a similar manner.
===================================================================*/

extern yakLib libDemo;
extern yakFont fntDemo;

//we'll define a rectangular region for the demo:
const screenCoordX_t CscxLeft   = 10;
const screenCoordY_t CscyTop    = 10;
const screenCoordX_t CscxRight  = 310;
const screenCoordY_t CscyBottom = 230;
const screenCoord_t  CscdSpace  = 65;

const screenCoordX_t CscxZoomLeft   = CscxLeft + 5;
const screenCoordY_t CscyZoomTop    = CscyTop + 80;
const screenCoordX_t CscxZoomRight  = ( CscxRight + CscxLeft ) / 2 - 5;
const screenCoordY_t CscyZoomBottom = CscyBottom - 15;

const screenCoordX_t CscxCollideLeft    = ( CscxRight + CscxLeft ) / 2 + 5;
const screenCoordY_t CscyCollideTop     = CscyZoomTop;
const screenCoordX_t CscxCollideRight   = CscxRight - 5;
const screenCoordY_t CscyCollideBottom  = CscyZoomBottom;
//these are background colors for the fill areas:
const color_t CcolFill = 25;
const color_t CcolZoom = 20;
const color_t CcolWhite = 15;

/*
CopyPage() copies all of one logical graphics page to another:
*/
void CopyPage(
  pageHandle_t pagPageTo,
  pageHandle_t pagPageFrom
)
{
  x_cp_vid_rect( 0, 0, ScrnLogicalPixelWidth, ScrnLogicalHeight,
    0, 0, pagPageFrom, pagPageTo,
    ScrnLogicalPixelWidth, ScrnLogicalPixelWidth );
}

#ifdef __386__
int random(
  int iLimit
)
{
  return rand() % iLimit;
}
#endif

/*
DrawZoomDemoFrame() draws a yakBitmap in the appropriate areas
*/
void DrawZoomDemoFrame(
  yakBitmap * pybm
)
{
  //get our starting coordinates
  static screenCoordX_t scx1 = CscxZoomLeft;
  static screenCoordX_t scx2 = CscxZoomRight;
  static screenCoordY_t scy1 = CscyZoomTop;
  static screenCoordY_t scy2 = CscyZoomBottom;

  //and our changes in those starting coordinates.
  static int iDeltaX1 = random( 5 );
  static int iDeltaX2 = random( 5 );
  static int iDeltaY1 = random( 5 );
  static int iDeltaY2 = random( 5 );

  //fill the area
  yakPrimitive::FillRectangle( HiddenPageOffs, CscxZoomLeft, CscyZoomTop, CscxZoomRight, CscyZoomBottom, CcolZoom );

  //show the zoomed yakBitmap in the prescribed region
  pybm->ShowZoomed(
    HiddenPageOffs,
    tMin( scx1, scx2 ), tMin( scy1, scy2 ),
    abs( scx2 - scx1 ), abs( scy2 - scy1 ), 
    yakBitmap::solid);

  //now update the counters and their deltas.  The idea is that each
  //relevant corner moves at a constant rate until it hits the edge of
  //the zooming rectangle.  When it hits, it changes its movement rate.
  scx1 = iBound( scx1 + iDeltaX1, CscxZoomLeft, CscxZoomRight );
  if ( ( scx1 == CscxZoomLeft ) ||
     ( scx1 == CscxZoomRight ) ) {
    iDeltaX1 = random( 5 ) - 2;
  }

  scx2 = iBound( scx2 + iDeltaX2, CscxZoomLeft, CscxZoomRight );
  if ( ( scx2 == CscxZoomLeft ) ||
     ( scx2 == CscxZoomRight ) ) {
    iDeltaX2 = random( 5 ) - 2;
  }

  scy1 = iBound( scy1 + iDeltaY1, CscyZoomTop, CscyZoomBottom );
  if ( ( scy1 == CscyZoomTop ) ||
     ( scy1 == CscyZoomBottom ) ) {
    iDeltaY1 = random( 5 ) - 2;
  }

  scy2 = iBound( scy2 + iDeltaY2, CscyZoomTop, CscyZoomBottom );
  if ( ( scy2 == CscyZoomTop ) ||
     ( scy2 == CscyZoomBottom ) ) {
    iDeltaY2 = random( 5 ) - 2;
  }
}


/*
DrawCollideDemoFrame() draws a demonstration of yakBitmap collision display and
detection with three yakBitmaps
*/
void DrawCollideDemoFrame(
  yakBitmap * pybm
)
{
  const int CiNumIcons = 3;

  //now set up our values; in this case, just to random numbers.  But
  //we can't do that in code without a little finagling, or it'll set
  //up the values every time we call the function, which we don't want.
  static screenCoordX_t ascxLeft[ CiNumIcons ]  = { CscxCollideLeft, CscxCollideLeft + 40, CscxCollideLeft + 80 };
  static screenCoordY_t ascyTop[ CiNumIcons ]   = { CscyCollideTop, CscyCollideTop + 40, CscyCollideTop + 80 };
  static screenCoordX_t ascxDelta[ CiNumIcons ] = { 1, 1, 1 };
  static screenCoordY_t ascyDelta[ CiNumIcons ] = { 1, 1, 1 };
  static color_t acolCollision[ CiNumIcons ] = { 0, 0, 0 };
  //here's our limits for the top left corner of the moving yakBitmap.
  screenCoordX_t scxFurthestRight = CscxCollideRight - pybm->iGetWidth();
  screenCoordY_t scyFurthestDown = CscyCollideBottom - pybm->iGetHeight();

  //clear the collision demo area:
  yakPrimitive::FillRectangle( HiddenPageOffs,
    CscxCollideLeft, CscyCollideTop,
    CscxCollideRight, CscyCollideBottom,
    CcolZoom
  );
  //now we'll start the loop, updating the yakBitmaps' positions and cropping
  //their motion as necessary.
  for ( int i = 0; i < CiNumIcons; ++i ) {
    //move this yakBitmap, bounding it to the right area and changing its
    //"delta" if it's at an edge
    ascxLeft[ i ] = iBound( ascxLeft[ i ] + ascxDelta[ i ],
      CscxCollideLeft, scxFurthestRight );
    if ( ( ascxLeft[ i ] == CscxCollideLeft ) ||
      ( ascxLeft[ i ] == scxFurthestRight ) ) {
        ascxDelta[ i ] = random( 5 ) - 2;
    }
    ascyTop[ i ] = iBound( ascyTop[ i ] + ascyDelta[ i ],
      CscyCollideTop, scyFurthestDown );
    if ( ( ascyTop[ i ] == CscyCollideTop ) ||
      ( ascyTop[ i ] == scyFurthestDown ) ) {
      ascyDelta[ i ] = random( 5 ) - 2;
    }
    //now check for a collision and draw one if necessary
    for ( int iTarget = 0; iTarget < CiNumIcons; ++iTarget ) {
      if ( iTarget != i ) {
        bFlag bfCollided = pybm->bfHitXY(
          ascxLeft[ i ], ascyTop[ i ],
          pybm, ascxLeft[ iTarget ], ascyTop[ iTarget ]
        );
        //if there is a collision, update the "collision color" and
        //change the deltas for this yakBitmap only ( since the other will get
        //updated when it checks )

        if ( bfCollided ) {
          acolCollision[ i ] = (color_t)random( 256 );
          ascxDelta[ i ] *= -1;
          ascyDelta[ i ] *= -1;
        }
      }
    }
    //finally, draw the yakBitmap, in its collided color if appropriate
    if ( acolCollision[ i ] != 0 ) {
      pybm->ShowCollision( HiddenPageOffs, ascxLeft[ i ], ascyTop[ i ], acolCollision[ i ] );
      acolCollision[ i ] = 0;
    }
    else {
      pybm->Show( HiddenPageOffs, ascxLeft[ i ], ascyTop[ i ] );
    }
  }
}



/*
DoYakBitmapDemo() displays a static yakBitmap in different modes.
*/
void DoYakBitmapDemo(
  void
)
{
  //load the logo yakBitmap, with collision and zooming options enabled.
  yakBitmap ybmLogo;
  bFlag bfOK = ybmLogo.bfLoad( "logo.yak", &libDemo, (yakBitmap::bitmapType)(yakBitmap::collidable | yakBitmap::zoomable) );
  ASSERT( bfOK );
  //blank out our demo area
  yakPrimitive::FillRectangle( VisiblePageOffs, CscxLeft, CscyTop, CscxRight, CscyBottom, CcolFill );

  //show the three different yakBitmap modes
  ybmLogo.Show( VisiblePageOffs, CscxLeft + 1, CscyTop + 1, yakBitmap::solid );
  ybmLogo.Show( VisiblePageOffs, CscxLeft + CscdSpace, CscyTop + 1, yakBitmap::masked );
  ybmLogo.ShowCollision( VisiblePageOffs, CscxLeft + 2 * CscdSpace, CscyTop + 1, CcolWhite );

  //now display the captions for each mode
  fntDemo.Printf( VisiblePageOffs, CscxLeft, CscyTop + CscdSpace, CcolWhite, "Normal" );
  fntDemo.Printf( VisiblePageOffs, CscxLeft + CscdSpace, CscyTop + CscdSpace, CcolWhite, "Masked" );
  fntDemo.Printf( VisiblePageOffs, CscxLeft + 2 * CscdSpace, CscyTop + CscdSpace, CcolWhite, "Collided" );
  fntDemo.Printf( VisiblePageOffs, CscxZoomLeft, CscyBottom - ( fntDemo.bCharHeight + 1 ), CcolWhite, "Zooming" );
  fntDemo.Printf( VisiblePageOffs, CscxCollideLeft, CscyBottom - ( fntDemo.bCharHeight + 1 ), CcolWhite, "Zooming" );

  //now copy all that onto the hidden page for double-buffering.
  CopyPage( HiddenPageOffs, VisiblePageOffs );
  //and start our loop for the zoomed yakBitmap
  loop {
    //wait for refresh
    x_wait_vsync();
    //draw our frames
    DrawZoomDemoFrame( &ybmLogo );
    DrawCollideDemoFrame( &ybmLogo );
    //then flip pages
    x_page_flip( 0, 0 );

    exitWhen( kbhit() );
  }

  //clean up the character
  getch();
}


/*
DoAnimBitmapDemo() displays an animated yakBitmap ( the grumjug ) to display
the usefulness of the animated yakBitmap structure.
*/
void DoAnimBitmapDemo(
  void
)
{
  //load an animated yakBitmap
  animBitmap anm;
  bFlag bfOK = anm.bfAddAllFrames( "grumjug", &libDemo, (yakBitmap::bitmapType)(yakBitmap::collidable | yakBitmap::zoomable) );
  ASSERT( bfOK );
  //blank out our demo area
  yakPrimitive::FillRectangle( VisiblePageOffs, CscxLeft, CscyTop, CscxRight, CscyBottom, CcolFill );
  //now display the captions for each mode
  fntDemo.Printf( VisiblePageOffs, CscxLeft, CscyTop + CscdSpace, CcolWhite, "Normal" );
  fntDemo.Printf( VisiblePageOffs, CscxLeft + CscdSpace, CscyTop + CscdSpace, CcolWhite, "Masked" );
  fntDemo.Printf( VisiblePageOffs, CscxLeft + 2 * CscdSpace, CscyTop + CscdSpace, CcolWhite, "Collided" );
  fntDemo.Printf( VisiblePageOffs, CscxZoomLeft, CscyBottom - ( fntDemo.bCharHeight + 1 ), CcolWhite, "Zooming" );
  fntDemo.Printf( VisiblePageOffs, CscxCollideLeft, CscyBottom - ( fntDemo.bCharHeight + 1 ), CcolWhite, "Zooming" );
  //now copy all that onto the hidden page for double-buffering.
  CopyPage( HiddenPageOffs, VisiblePageOffs );

  loop {
    //wait for refresh
    x_wait_vsync();
    //blank out the backgrounds for the three yakBitmap modes
    yakPrimitive::FillRectangle( HiddenPageOffs, CscxLeft, CscyTop, CscxRight, CscyTop + 2 + anm.tIThisFrame.ptCurrent()->iGetHeight(), CcolFill );
    //show the three different yakBitmap modes
    anm.Show( HiddenPageOffs, CscxLeft + 1, CscyTop + 1, yakBitmap::solid );
    anm.Show( HiddenPageOffs, CscxLeft + CscdSpace, CscyTop + 1, yakBitmap::masked );
    anm.ShowCollision( HiddenPageOffs, CscxLeft + 2 * CscdSpace, CscyTop + 1, CcolWhite );
    //show our demo frames...
    DrawZoomDemoFrame( anm.tIThisFrame.ptCurrent() );
    DrawCollideDemoFrame( anm.tIThisFrame.ptCurrent() );
    //then flip pages
    x_page_flip( 0, 0 );

    //now advance the animBitmap so we get fluid motion!
    anm.Advance();

    exitWhen( kbhit() );
  }

  //clean up the character
  getch();
}


/*
main()
*/
void main(
  void
)
{
  InitDemoObjects();
  SpewText( "icondemo.txt" );
  SetUpGraphics();
  DoYakBitmapDemo();
  DoAnimBitmapDemo();
  QuitFromGraphics();
}



