#include "yakicons.h"
#include "demorts.h"

#ifdef __WATCOMC__
int random( int iLimit ) { return rand() % iLimit; }
#endif

//these are defined in demorts.cpp
extern yakLib libDemo;
extern yakFont fntDemo;

/*
GetKeys moves the actor according to the keys being pressed at the time.
*/
void getKeys(
  facingActor * pfct
)
{
  int iDeltaX =0;
  int iDeltaY=0;
  if ( keyboard.bfIsPressed( yakKeyboard::upArrow ) != 0 ) {
    iDeltaY -= 4;
  }
  if ( keyboard.bfIsPressed( yakKeyboard::downArrow ) != 0 ) {
    iDeltaY += 4;
  }
  if ( keyboard.bfIsPressed( yakKeyboard::leftArrow ) != 0 ) {
    iDeltaX -= 4;
  }
  if ( keyboard.bfIsPressed( yakKeyboard::rightArrow ) != 0 ) {
    iDeltaX += 4;
  }
  if ( ( iDeltaX != 0 ) ||
     ( iDeltaY != 0 ) ) {
    pfct->AddCommand( scriptLine::relativeMove, iDeltaX, iDeltaY, 0, 0 );
  }
  else {
    pfct->AddCommand( scriptLine::doNothing, 0, 0, 0, 0 );
  }
}


/*
DoMapDemo() loads a sample map, populates it with "tree" actors and
"grumjug" facingActors, and has the grums run about.
*/
void DoMapDemo(
  void
)
{
  //we're letting the player control the grumjug through the keyboard,
  //so initialize the keyboard first.
  keyboard.Install();
  //we'll also be doing draws which could overflow the page, so make sure
  //the clipping rectangle is set properly!
  yakPrimitive::SetClippingRectangle( 0, 0, ScrnLogicalPixelWidth, ScrnLogicalHeight );
  //load the map first
  animap mapDemo;
  bFlag bfOK;
  bfOK = mapDemo.bfLoad( "maptest.map", &libDemo, &libDemo );
  ASSERT( bfOK );

  //now set up our grumjug actors
  //load the animBitmaps for the facingActors
  directionTable  dirGrumjug;
  dirGrumjug.Load( "", "", "", "", "lgrumjug", "cgrumjug", "grumjug", "", "", "", &libDemo );

  //set up our demo facingActor.  It's twoFace to avoid drawing a different
  //animBitmap for each diagonal.
  facingActor fctPrimary;
  fctPrimary.AssignNewDirectionTable( &dirGrumjug, facingActor::twoFace );
  fctPrimary.PutToMap( &mapDemo, 1, 1 );

  //now set up our tree actors...
  //load an animBitmap for the image of the tree
  animBitmap anmTree;
  bfOK = anmTree.bfAddAllFrames( "tree", &libDemo );
  //now initialize an array of tree actors.  We're doing this so that
  //the tree "actors", which won't move, will be placed randomly on the
  //map to add interesting variety rather than affixing them to a set
  //position in each tile.
  const CiNumTrees = 75;
  actor * apactTrees[ CiNumTrees ];
  for ( int i = 0; i < CiNumTrees; ++i ) {
    apactTrees[ i ] = new actor;
  }
  i = 0;
  loop {
    //initialize it to the image
    apactTrees[ i ]->Initialize( &anmTree );
    //now calculate new random coordinates for the tree's location
    mapCoordX_t mcxMapWidth = mapDemo.mcdSquareWidth * mapDemo.tcxWidth;
    mapCoordY_t mcyMapHeight = mapDemo.mcdSquareWidth * mapDemo.tcyHeight;
    mapCoordX_t mcxNew = random( mcxMapWidth );
    mapCoordY_t mcyNew = random( mcyMapHeight );
    //if the new location of the tree's bottom is a grass square, put it there!
    if ( mapDemo.terTerrainType(
      iRollover( mcxNew + anmTree.tIThisFrame.ptCurrent()->iGetWidth(), mcxMapWidth ),
      iRollover( mcyNew + anmTree.tIThisFrame.ptCurrent()->iGetHeight(), mcyMapHeight ) ) == 0 ) {
      apactTrees[ i ]->PutToMap( &mapDemo, mcxNew, mcyNew );
      ++i;
    }
    exitWhen( i >= CiNumTrees );
  }


  //draw the first two frames
  const int CiDrawWidth = 6;
  const int CiDrawHeight = 4;
  const screenCoordX_t CscxShift = 0;
  const screenCoordY_t CscyShift = 0;

  mapDemo.DrawAndAdvance( HiddenPageOffs, CscxShift, CscyShift,
    fctPrimary.tcxLeft() - 2, fctPrimary.tcyTop() - 1,
    CiDrawWidth, CiDrawHeight,
    (yakBitmap::showType)(yakBitmap::solid | yakBitmap::clipped), (yakBitmap::showType)(yakBitmap::masked | yakBitmap::clipped) );
  x_page_flip( 24 + fctPrimary.iInTileX(), 24 + fctPrimary.iInTileY() );
  mapDemo.DrawAndAdvance( HiddenPageOffs, CscxShift, CscyShift,
    fctPrimary.tcxLeft() - 2, fctPrimary.tcyTop() - 1,
    CiDrawWidth, CiDrawHeight,
    (yakBitmap::showType)(yakBitmap::solid | yakBitmap::clipped), (yakBitmap::showType)(yakBitmap::masked | yakBitmap::clipped) );

  while ( !keyboard.bfIsPressed( yakKeyboard::escape ) ) {
    //draw themap and flip pages
    mapDemo.Draw( HiddenPageOffs, CscxShift, CscyShift,
      fctPrimary.tcxLeft() - 2, fctPrimary.tcyTop() - 1,
      CiDrawWidth, CiDrawHeight,
      (yakBitmap::showType)(yakBitmap::solid | yakBitmap::clipped), (yakBitmap::showType)(yakBitmap::masked | yakBitmap::clipped) );
    x_page_flip( fctPrimary.iInTileX() + CscxShift, fctPrimary.iInTileY() + CscyShift );

    //print out statistics on the primary facingActor
    fntDemo.BgPrintf( SplitScrnOffs, 10, 10, 15, 20, "X %d   ", fctPrimary.mcxLeft() );
    fntDemo.BgPrintf( SplitScrnOffs, 10, 30, 15, 20, "Y %d   ", fctPrimary.mcyTop() );

    //advance the map
    mapDemo.Advance();

    //get input for moving the primary facingActor
    getKeys( &fctPrimary );
  }

  //clean up; remove keyboard driver
  keyboard.Remove();
  //and delete our tree actors
  for ( i = 0; i < CiNumTrees; ++i ) {
    SafelyFree( apactTrees[ i ] );
  }
}

/*
main() runs the demo
*/
void main(
  void
)
{
  #ifndef __386__
  emsBlock::bfInit();
  #endif
  InitDemoObjects();
  SpewText( "mapdemo.txt" );
  SetUpGraphics();
  x_show_splitscreen();
  DoMapDemo();
  QuitFromGraphics();
}
