#include "yakicons.h"
#include "demorts.h"
#include <conio.h>

extern yakPalette palDemo;
extern yakFont fntDemo;

/*
ShowUserMessage() just blanks the split screen and shows a message.
*/
void ShowUserMessage(
  const char * CpcMessage
)
{
  yakPrimitive::FillRectangle( SplitScrnOffs, 0, 0, ScrnPhysicalPixelWidth, fntDemo.bCharHeight, 0 );
  fntDemo.Printf( SplitScrnOffs, 0, 0, 15, CpcMessage );
}


/*
RotateDemo() just rotates the palette and waits for a keypress
*/
void RotateDemo(
  yakPalette * ppal
)
{
  ShowUserMessage( "Simple palette rotation.  Press a key." );
  loop {
    ppal->Rotate( yakPalette::down );
    ppal->Put();
    exitWhen( kbhit() );
  }
  getch();
}


/*
TweenDemo() tweens between two palettes, rotating as it does
*/
void TweenDemo(
  yakPalette * ppal1,
  yakPalette * ppal2,
  yakPalette::rotateDirection eDirection,
  int iNumberOfSteps
)
{
  yakPalette palTweened;
  ShowUserMessage( "Tweening to a different palette. " );
  #define TWEEN_STEPS 64
  for ( int i = 0; i < TWEEN_STEPS; ++i ) {
    ppal1->Rotate( eDirection );
    ppal2->Rotate( eDirection );
    palTweened.SetToTween( ppal1, ppal2, i, TWEEN_STEPS );
    palTweened.Put();
  }
}

/*
DoPaletteDemo() runs through a few simple palette tricks
*/
void DoPaletteDemo(
  void
)
{
  //first show the split screen so we can communicate with the user
  x_show_splitscreen();
  //now draw concentric rectangles
  for( int i = 0; i < SplitScrnScanLine / 2; ++i ) {
    yakPrimitive::FillRectangle( VisiblePageOffs, i, i, ScrnPhysicalPixelWidth - i, SplitScrnScanLine - i, i  );
  }
  //now do our rotation demo
  RotateDemo( &palDemo );
  //now set to tween colors; we'll tween to a greyscale palette
  yakPalette palGrey;
  for ( i = 0; i < 255; ++i ) {
    palGrey.SetColor( i, i % 64, i % 64, i % 64 );
  }
  TweenDemo( &palDemo, &palGrey, yakPalette::up, 64 );
  //rotate the palette again...
  RotateDemo( &palGrey );
  //now tween back
  TweenDemo( &palGrey, &palDemo, yakPalette::down, 32 );
  //now rotate again
  RotateDemo( &palDemo );
  //now do a fade
  ShowUserMessage( "Palette fade to white." );
  for ( i = 0; i < 32; ++i ) {
    palDemo.Fade( 2 );
    palDemo.Put();
  }

}


/*
main() sets up graphics and executes the palette demo
*/
void main(
  void
)
{
  InitDemoObjects();
  SpewText( "paldemo.txt" );
  SetUpGraphics();
  DoPaletteDemo();
  QuitFromGraphics();
}