#include "yakicons.h"
#pragma hdrstop
#include "platform.h"

player * firstPlayer = NULL, *secondPlayer = NULL;


/*
default constructor for platformBeasts.
*/
platformBeast::platformBeast(
	void
)
{
	iDeltaX =
		iAlignment =
		iDamage =
		iDeltaY =
		iHitPoints=
		iMoveIncrement =
		iJumpIncrement = 0;
	iGravity = 2;
};


/*
Go() changes the platformBeast's momentum based on the command given.
*/
void platformBeast::Go(
	command myCommand
)
{
	switch( myCommand ) {
		case goLeft :
			iDeltaX = -iMoveIncrement;
		break;
		case goRight :
			iDeltaX = iMoveIncrement;
		break;
		case jump :
			if ( pmapOwning->terTerrainType(
				iRollover( ( mcxRight() + mcxLeft() ) / 2, pmapOwning->mcxWidth() ),
				iRollover( mcyBottom() + 1, pmapOwning->mcyHeight() ) ) == 1 ) {
				iDeltaY = -iJumpIncrement;
			}
		break;
	}
}

void platformBeast::Advance(
	void
)
{
	iDeltaY += iGravity;

	//first check to see if travelling down will put character in a solid square.
	if ( pmapOwning->terTerrainType(
		iRollover( ( mcxRight() + mcxLeft() ) / 2 + iDeltaX, pmapOwning->mcxWidth() ),
		iRollover( mcyBottom() + iDeltaY, pmapOwning->mcyHeight() ) ) == 1 ) {
			RemoveFromMap();
			mcy = ( tcyTop() + 1 ) * pmapOwning->mcdSquareWidth - ptCurrent()->iGetHeight() + 1;
			iDeltaY = 0;
			PutToMap( pmapOwning, mcx, mcy );
	}

	//now check to see if moving up (jumping, etc) will put you in a solid square
	if ( pmapOwning->terTerrainType(
		iRollover( ( mcxRight() + mcxLeft() ) / 2 + iDeltaX, pmapOwning->mcxWidth() ),
		iRollover( mcyTop() + iDeltaY, pmapOwning->mcyHeight() ) ) == 1 ) {
			RemoveFromMap();
			mcy = ( tcyTop() * pmapOwning->mcdSquareWidth );
			iDeltaY = 0;
			PutToMap( pmapOwning, mcx, mcy );
	}

	//now check to see if moving right will put you in a solid square.
	if ( pmapOwning->terTerrainType(
		iRollover( mcxRight() + iDeltaX, pmapOwning->mcxWidth() ),
		iRollover( ( mcyTop() + mcyBottom() ) / 2 + iDeltaY, pmapOwning->mcyHeight() ) ) == 1 ) {
			RemoveFromMap();
			mcx = ( tcxLeft() + 1 ) * pmapOwning->mcdSquareWidth - ptCurrent()->iGetWidth();
			iDeltaX = 0;
			PutToMap( pmapOwning, mcx, mcy );
	}

	//now check to see if moving left will put you in a solid square (whew!)
	if ( pmapOwning->terTerrainType(
		iRollover( mcxLeft() + iDeltaX, pmapOwning->mcxWidth() ),
		iRollover( ( mcyTop() + mcyBottom() ) / 2 + iDeltaY, pmapOwning->mcyHeight() ) ) == 1 ) {
			RemoveFromMap();
			mcx = tcxLeft() * pmapOwning->mcdSquareWidth;
			iDeltaX = 0;
			PutToMap( pmapOwning, mcx, mcy );
	}

	//if you're doing anything, add that command
	if ( ( iDeltaX != 0 ) ||
		 ( iDeltaY != 0 ) ) {
		AddCommand( scriptLine::relativeMove, iDeltaX, iDeltaY, 0, 0 );
	}

	//otherwise, do nothing
	else {
		AddCommand( scriptLine::doNothing, 0, 0, 0, 0 );
	}

	//call the usual Advance function
	facingActor::Advance();

	//if you're a baddie,
	if ( iAlignment < 0 ) {
		//if you're touching the player
		if ( bfHitActor( firstPlayer ) )	{
			firstPlayer -> iHitPoints -= iDamage;
			firstPlayer -> colCollisionColor = 15;
			iHitPoints -= firstPlayer->iDamage;
			colCollisionColor = 31;
		}

		int hitMissile = 0;
		//if you're hit with a missile
		if ( ( hitMissile = firstPlayer->iHitWithMissileType( this ) ) != 0 ) {
			iHitPoints -= firstPlayer->apmisMissiles[ hitMissile - 1 ]->iDamage;
			colCollisionColor = 47;
		}
	}
	if ( iHitPoints <= 0 ) {
		Die();
	}
}


/*
Die() removes the beast from the map.
*/
void platformBeast::Die(
	void
)
{
	RemoveFromMap();
}


/*
default constructor for missiles.
*/
missile::missile(
	void
)
{
	iGravity =
		iRange =
		iMissileNumber =
		iDamage = 0;
	pplyOwning = NULL;
}


/*
redefines Advance().
*/
void missile::Advance(
	void
)
{
	//Advance as any other platform beast
	platformBeast::Advance();

	//decrease range.  If we're out of range, remove from map and reset
	//the player's missile bay
	--iRange;
	if ( iRange == 0 )	{
		RemoveFromMap();
		if ( pplyOwning != NULL ) {
			pplyOwning->apmisMissiles[ iMissileNumber ] = NULL;
		}
		delete this;
	}
}


/*
default constructor for players.
*/
player::player(
	void
) :
	platformBeast()
{
	for ( int i = 0; i < MAX_MISSILES; ++i ) {
		apmisMissiles[ i ] = NULL;
	}
}


/*
fireMissile() launches a player's missile in the appropriate direction.
*/
void player::FireMissile(
	void
)
{
	//find the next blank missile spot, if any
	int iMissileCount = 0;
	loop {

		exitWhen( ( apmisMissiles[ iMissileCount ] == NULL ) ||
			( iMissileCount >= iNumMissiles ) );

		++iMissileCount;
	}

	//if we've found a valid spot
	if ( iMissileCount != iNumMissiles ) {
		//make a new missile
		apmisMissiles[ iMissileCount ] = pmisMissileGenerator( iMissileType );
		missile * newMissile = apmisMissiles[ iMissileCount ];
		newMissile->drcDirection = drcDirection;
		newMissile->iMissileNumber = iMissileCount;

		//now set up the momentum depending on our direction
		switch ( drcDirection )	{
			case facingActor::east:
			case facingActor::southeast:
			case facingActor::northeast:
				newMissile->iDeltaX = iMoveIncrement;
			break;
			case facingActor::west:
			case facingActor::southwest:
			case facingActor::northwest:
				newMissile->iDeltaX = -iMoveIncrement;
			break;
		}

		//and put the missile on the map
		newMissile->PutToMap( pmapOwning, mcx, mcy);
	}
}


/*
hitWithMissile() returns the missile number that struck the target, if any.
*/
int player::iHitWithMissileType(
	platformBeast * target
)
{
	for ( int iMissileCount = 0; iMissileCount < iNumMissiles; ++iMissileCount ) {
		if ( ( apmisMissiles[ iMissileCount ] != 0 ) &&
			 ( apmisMissiles[ iMissileCount ]->bfHitActor( target ) == true ) ) {
			return iMissileCount + 1;  //so zero is "no hit"
		}
	}
	return 0;
}


/*
Advance() redefines platformBeast::Advance().
*/
void crawler::Advance(
	void
)
{
	//now execute the command
	if ( drcDirection == facingActor::east ) {
		Go( platformBeast::goRight );
	}
	else {
		Go( platformBeast::goLeft );
	}

  terrainType_t terNextBody = 0;
  terrainType_t terNextFloor = 0;
  if ( iDeltaX < 0 ) {
		terNextBody = pmapOwning->terTerrainType(
			iRollover( mcxLeft() + iDeltaX, pmapOwning->mcxWidth() ),
			iRollover( mcyBottom() - 1, pmapOwning->mcyHeight() )
		);
  }
  else {
		terNextBody = pmapOwning->terTerrainType(
			iRollover( mcxRight() + iDeltaX, pmapOwning->mcxWidth() ),
			iRollover( mcyBottom() - 1, pmapOwning->mcyHeight() )
		);
  }
	terNextFloor = pmapOwning->terTerrainType(
		iRollover( ( mcxRight() + mcxLeft() ) / 2, pmapOwning->mcxWidth() ),
		iRollover( mcyBottom() + iDeltaY + 1, pmapOwning->mcyHeight() )
	);

	//if the next square isn't floor
	if ( ( terNextBody & FLOOR ) ||
		!( terNextFloor & FLOOR ) ) {
		//reverse direction
		if ( drcDirection == facingActor::east ) {
			drcDirection = facingActor::west;
		}
		else {
			drcDirection = facingActor::east;
		}
	}

	//now execute the command
	if ( drcDirection == facingActor::east ) {
		Go( platformBeast::goRight );
	}
	else {
		Go( platformBeast::goLeft );
	}


	//and Advance like any other good platformBeast
	platformBeast::Advance();
}

