#include "yakicons.h"

#include "demorts.h"
#include "sb1misc.h"

extern yakLib libDemo;
extern yakFont fntDemo;

#ifdef __386__
int random( int iLimit ) {
  return rand() % iLimit;
}
#endif

/*
GetKeyCommands() gets input from the player and moves the character
appropriately
*/
void GetKeyCommands(
  grumPlayer * pplayer
)
{
  if ( pplayer->iDeltaY == 0 ) {
    pplayer->iDeltaX = 0;
	}
	if ( keyboard.bfIsPressed( yakKeyboard::upArrow ) ) {
		pplayer->Go( platformBeast::jump );
	}
	if ( keyboard.bfIsPressed( yakKeyboard::leftArrow ) ) {
		pplayer->Go( platformBeast::goLeft );
	}
	if ( keyboard.bfIsPressed( yakKeyboard::rightArrow ) ) {
		pplayer->Go( platformBeast::goRight );
	}
	if ( keyboard.bfIsPressed( yakKeyboard::space ) ) {
		pplayer->FireMissile();
	}
	if ( keyboard.bfIsPressed( yakKeyboard::downArrow ) ) {
		pplayer->iDeltaX = 0;
	}
}


/*
DoSmartRefreshDemo() shows a map made of blocks and sky, with a hopping
grumjug ( player-controlled ) launching spiked clubs at blobs.
*/
void DoSmartRefreshDemo(
  void
)
{
  keyboard.Install();
  //set up our player
  grumPlayer player1;
  extern player * firstPlayer;
  firstPlayer = &player1;
  //load the platform game icons;
  loadIcons();
	animap mapDemo( "beet.map", &libDemo, &libDemo );

  //set up the clipping rectangle
  yakPrimitive::SetClippingRectangle( 0, 0, ScrnLogicalPixelWidth, ScrnLogicalHeight );
	//set up player1 object
	player1.AssignNewDirectionTable( &( adirBeetIcons[ PLAYER ] ), facingActor::eightFace );
	player1.PutToMap( &mapDemo, 23, 23 );

	//set up array of blobs
  const int CiNumBlobs = 10;
	blob *aptBlobs[ CiNumBlobs ];
	for ( int iBlobCount = 0; iBlobCount < CiNumBlobs; ++iBlobCount ) {
		aptBlobs[ iBlobCount ] = new blob;
		aptBlobs[ iBlobCount ]->PutToMap(
      &mapDemo,
			random( mapDemo.mcxWidth() ),
			random( mapDemo.mcyHeight() )
		);
	}

  const screenCoordX_t CscxShift = 0;
	const screenCoordY_t CscyShift = 0;
  const int CiDrawWidth = 8;
  const int CiDrawHeight = 5;
	//draw the first two frames
	mapDemo.DrawAndAdvance( HiddenPageOffs, CscxShift, CscyShift,
		player1.tcxLeft() - 3, player1.tcyTop() - 1,
		CiDrawWidth, CiDrawHeight,
		( yakBitmap::showType )( yakBitmap::solid | yakBitmap::clipped ),
		( yakBitmap::showType )( yakBitmap::masked | yakBitmap::clipped ) );
	x_page_flip( player1.iInTileX(), player1.iInTileY() );
	mapDemo.DrawAndAdvance( HiddenPageOffs, CscxShift, CscyShift,
		player1.tcxLeft() - 3, player1.tcyTop() - 1,
		CiDrawWidth, CiDrawHeight,
		( yakBitmap::showType )( yakBitmap::solid | yakBitmap::clipped ),
		( yakBitmap::showType )( yakBitmap::masked | yakBitmap::clipped ) );

	//main loop
	loop {

		int iDrawX = player1.tcxLeft() - 3;
		int iDrawY = player1.tcyTop() - 1;

		int iFlipX = player1.iInTileX();
		int iFlipY = player1.iInTileY();

		int iDeltaX = 0;
		int iDeltaY = 0;

		//if we've moved
		if ( ( iDrawX != mapDemo.tcxLastDrawLeft ) ||
			 ( iDrawY != mapDemo.tcyLastDrawTop ) )	{

			mapCoord_t mcdOldX = player1.mcxLeft();
			mapCoord_t mcdOldY = player1.mcyTop();

			//do the refresh without resetting the refresh counters
			mapDemo.SmartRefresh( HiddenPageOffs, iDrawX, iDrawY, false,
				( yakBitmap::showType )( yakBitmap::solid | yakBitmap::clipped ),
				( yakBitmap::showType )( yakBitmap::masked | yakBitmap::clipped ) );

			//flip the page
			x_page_flip( iFlipX, iFlipY );

			//either way, get the player's input and advance the map
			GetKeyCommands( &player1 );
			mapDemo.Advance();

			iDeltaX = player1.mcxLeft() - mcdOldX;
			iDeltaY = player1.mcyTop() - mcdOldY;

		}

		//if we haven't moved map squares
		{

			//refresh the map, updating the refresh counters
			mapDemo.SmartRefresh( HiddenPageOffs, iDrawX, iDrawY, true,
				( yakBitmap::showType )( yakBitmap::solid | yakBitmap::clipped ),
				( yakBitmap::showType )( yakBitmap::masked | yakBitmap::clipped ) );

			//flip to show the new page
			x_page_flip( iFlipX + iDeltaX, iFlipY + iDeltaY );

			//then write the player's statistics
			fntDemo.BgPrintf( SplitScrnOffs, 10, 10, 15, 20,
				"MapX %d***Mapy %d    ", player1.tcxLeft(), player1.tcyTop() );
			fntDemo.BgPrintf( SplitScrnOffs, 10, 20, 15, 20,
				"X %d***Y %d    ", player1.mcxLeft(), player1.mcyTop() );
			fntDemo.BgPrintf( SplitScrnOffs, 10, 30, 15, 20,
				"Hit Points:    %d    ", player1.iHitPoints );
		}

		//either way, get the player's input and advance the map
		GetKeyCommands( &player1 );
		mapDemo.Advance();

		exitWhen( keyboard.bfIsPressed( yakKeyboard::escape ) != 0 );

	}

	//clean up the blobs
	for ( iBlobCount = 0; iBlobCount < CiNumBlobs; ++iBlobCount ) {
		delete aptBlobs[ iBlobCount ];
	}

	//remove the keyboard driver
	keyboard.Remove();

}

/*
main()
*/
void main(
  void
)
{
	InitDemoObjects();
  SpewText( "smrtdemo.txt" );
  SetUpGraphics();
  x_show_splitscreen();
  DoSmartRefreshDemo();
  QuitFromGraphics();
}
